from datetime import datetime
from typing import Optional

from shortuuid import ShortUUID
from sqlmodel import Field
from sqlmodel import SQLModel

s = ShortUUID(alphabet="0123456789")


def generate_suid():
    return s.random(length=21)


class UserBase(SQLModel):
    id: str = Field(primary_key=True, min_length=21, max_length=21)
    email: str = Field(unique=True, index=True)
    verified: bool = Field(default=False)
    picture: Optional[str] = Field(default=None)

    is_active: bool = Field(default=True)


class UserCreate(UserBase):
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime = Field(default_factory=datetime.utcnow)


class User(UserBase, table=True):
    created_at: datetime = Field(nullable=False)
    updated_at: datetime = Field(nullable=False)


class ActivityBase(SQLModel):
    type: str = Field(nullable=False)
    where: str = Field(nullable=False)
    source: str = Field(nullable=False)
    browser: Optional[str] = Field(default=None)


class ActivityCreate(ActivityBase):
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime = Field(default_factory=datetime.utcnow)


class Activity(ActivityBase, table=True):
    id: str = Field(default_factory=generate_suid, primary_key=True, min_length=21, max_length=21)

    created_at: datetime = Field(nullable=False)
    updated_at: datetime = Field(nullable=False)



from typing import Optional

from loguru import logger
from sqlmodel import create_engine
from sqlmodel import select
from sqlmodel import Sequence
from sqlmodel import Session
from sqlmodel import SQLModel

from app.lib import models
from app.lib.settings import settings

engine = create_engine(
    settings.DB_URI,
    echo=True,
    connect_args={"check_same_thread": False}
)


def create_db_and_tables():
    SQLModel.metadata.create_all(engine)
    logger.info("Created tables")


# Dependency
def get_db():
    with Session(engine, expire_on_commit=False) as session:
        yield session


def create_user(conn: Session, user: models.UserCreate) -> models.User:
    db_user = models.User.model_validate(user)
    conn.add(db_user)
    conn.commit()
    conn.refresh(db_user)
    return db_user


def read_users(conn: Session, skip: int = 0, limit: int = 100) -> Sequence[models.User]:
    return conn.exec(select(models.User).offset(skip).limit(limit)).all()


def read_user_by_email(conn: Session, email: str) -> Optional[models.User]:
    return conn.exec(select(models.User).where(models.User.email == email)).first()


def create_activity(conn: Session, activity: models.ActivityCreate) -> models.Activity:
    db_activity = models.Activity.model_validate(activity)
    conn.add(db_activity)
    conn.commit()
    conn.refresh(db_activity)
    return db_activity



from os import getenv
from typing import Optional

from loguru import logger
from pydantic_settings import BaseSettings


class Settings(BaseSettings):
    ENV: Optional[str] = None
    DB_URI: str = "sqlite:///./sql_app.db"

    class Config:
        # Two envs: dev (local) and alpha (production)
        env_type = getenv("PEGASUS_ENV", "")

        env_file = "./.env.prod" if env_type == "alpha" else ".env.dev"
        logger.debug(f"Using env file: {env_file}")


settings = Settings()



from click import group, option
from loguru import logger
from subprocess import Popen, PIPE

from .browser import get_page_source
from .utils import get_host


@group(name='web', help="Advanced web utilities")
def web():
    pass


@web.command(help="View a web page")
@option('-l', '--link', type=str, required=True, prompt=True,
        help="Link to the web page")
def view(link: str):
    logger.debug("web view")
    source = get_page_source(link)
    print(source)


@web.command(help="Renders a web page (Ctrl + C to exit)")
@option('-l', '--link', type=str, required=True, prompt=True,
        help="Link to the web page")
def render(link: str):
    logger.debug("web open")
    source = get_page_source(link)
    out_file = f"{get_host(link)}.html"
    with open(out_file, "w") as f:
        f.write(source)
    command = f"lynx {out_file}"
    p = Popen(command, shell=True)
    p.wait()



from playwright.sync_api import sync_playwright


def get_page_source(url: str) -> str:
    with sync_playwright() as p:
        browser = p.chromium.launch()
        context = browser.new_context()
        page = context.new_page()
        page.goto(url)
        page_source = page.content()
        browser.close()
        return page_source


if __name__ == "__main__":
    print(get_page_source("https://zeffmuks.com"))



from urllib.parse import urlparse
from re import sub as re_sub


def get_host(link: str) -> str:
    host = urlparse(link).netloc
    host = re_sub(r"^www\.", "", host)
    host = host.replace("/", "")
    host = re_sub(r"\.co.*", "", host)
    return host


if __name__ == "__main__":
    print(get_host("https://www.google.com/api/v1/search?q=test"))
    print(get_host("https://zeffmuks.com/projects/project.json"))



metadata = {
    "/": {
        "title": "Pegasus - Next Generation Web Framework | Home",
        "description": "Pegasus combines the best of both worlds, a modern web framework with a powerful backend",
        "keywords": "Pegasus, Web, Python, TypeScript",
        "robots": "index, follow",
        "og_type": "website",
        "twitter_card": "summary_large_image",
        "author": "Zeff Muks",
    },
    "/error": {
        "robots": "noindex, nofollow",
        "title": "Pegasus - Error",
        "description": "An error occurred",
    }
}



from fastapi import APIRouter
from fastapi import Depends

from ..lib.auth import get_current_user
from ..lib.auth import TokenInfo

router = APIRouter(
    prefix="/users",
    tags=["users"],
)


@router.get("/me", response_model=TokenInfo)
def post_me(
        current_user: TokenInfo = Depends(get_current_user)
):
    """Get the current user (it creates a new user if token is valid and email is verified)"""
    return current_user



from os import getenv
from os import path as os_path
from typing import List

from boto3 import client as boto3_client
from click import Path, argument, group, option
from loguru import logger

from pd.__model__.model import PD_CONFIG_FILE, Instance

from ..config.config import add_instances, load_pdconfig, remove_instance, update_instance

AWS_ACCESS_KEY_ID = getenv("AWS_ACCESS_KEY_ID")
AWS_SECRET_ACCESS_KEY = getenv("AWS_SECRET_ACCESS_KEY")
AWS_DEFAULT_REGION = getenv("AWS_DEFAULT_REGION")


@group(name="aws", help="Manage AWS resources")
def aws():
    logger.debug("aws")
    if (
        AWS_ACCESS_KEY_ID is None
        or AWS_ACCESS_KEY_ID == ""
        or AWS_SECRET_ACCESS_KEY is None
        or AWS_SECRET_ACCESS_KEY == ""
        or AWS_DEFAULT_REGION is None
        or AWS_DEFAULT_REGION == ""
    ):
        print("Please set AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, and AWS_DEFAULT_REGION")
    return


@aws.command(help="Upload files to S3")
@option("-b", "--bucket", type=str, required=True, prompt=True, help="S3 bucket name")
@option("-d", "--directory", type=str, default="", help="Directory to upload files to")
@argument("filepaths", nargs=-1, type=Path(exists=True), required=True)
def s3(bucket: str, directory: str, filepaths: tuple[str, ...]):
    logger.debug("aws upload")

    for filepath in filepaths:
        logger.info(f"Uploading {filepath} to S3")
        s3_client = boto3_client(
            "s3",
            aws_access_key_id=AWS_ACCESS_KEY_ID,
            aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
            region_name=AWS_DEFAULT_REGION,
        )

        filename = os_path.basename(filepath)
        filebytes = open(filepath, "rb").read()

        s3_client.put_object(Bucket=bucket, Key=f"{directory}/{filename}", Body=filebytes)


@aws.command(help="Launch an EC2 instance")
@option("-n", "--name", type=str, required=True, prompt=True, help="Name of the EC2 instance e.g. ec2-test")
@option("-c", "--count", type=int, default=1, help="Number of instances to launch")
@option("-p", "--project", type=str, default="", help="Project name e.g /path/to/project")
def launch(name, count, project):
    logger.debug("ec2 launch")

    if count > 1:
        logger.warning(f"{count} instances are being launched...")
        input("Press Enter to continue...")
    if project != "":
        if not os_path.isabs(project):
            logger.error(f"Project path {project} is not absolute")
            return
        if not os_path.exists(project):
            logger.error(f"Project {project} does not exist on disk")
            return

    # Load config.
    config = load_pdconfig()

    if "ec2" not in config or len(config["ec2"]) == 0:
        print(f"EC2 config not found, please update {PD_CONFIG_FILE}[ec2]")
        return

    ec2_config = config["ec2"]
    if all(k in ec2_config for k in ("launch-template-id", "key-pair-path")) is False:
        print(f"EC2 config is missing fields, please update {PD_CONFIG_FILE}[ec2]")
        return

    launch_template_id = config["ec2"]["launch-template-id"]
    key_pair_name = config["ec2"]["key-pair-path"].split("/")[-1]
    if key_pair_name.endswith(".pem"):
        key_pair_name = key_pair_name[:-4]

    instances = launch_instances(
        launch_template_id=launch_template_id, key_pair_name=key_pair_name, instance_name=name, count=count
    )
    if len(instances) == 0:
        print("Failed to launch EC2 instances")
        return

    if count == 1:
        print(f"Launched 1 EC2 Instance: {instances[0]}")
    else:
        print(f"Launched {count} EC2 Instances: {instances}")

    add_instances(instances=instances, project=None if project == "" else project)


@aws.command(help="Connect to an EC2 instance")
@option(
    "-i",
    "--instance-id",
    type=str,
    required=True,
    prompt=True,
    help="ID of the EC2 instance to connect to e.g. i-1234567890abcdef",
)
def connect(instance_id):
    logger.debug("ec2 connect")


@aws.command(help="Allocate an Elastic IP Address")
@option(
    "-i",
    "--instance-id",
    type=str,
    required=True,
    prompt=True,
    help="ID of the EC2 instance to allocate an EIP for e.g. i-1234567890abcdef",
)
@option("-p", "--project", type=str, default="", help="Project name e.g /path/to/project")
def allocate(instance_id, project):
    logger.debug("ec2 allocate")

    public_ip = create_and_associate_eip(instance_id)
    if public_ip == "":
        print("Failed to allocate EIP")
        return

    print(f"Allocated EIP: {public_ip}")

    update_instance(instance=Instance(id=instance_id, ip=public_ip), project=None if project == "" else project)


@aws.command(help="Terminate an EC2 instance")
@option(
    "-i",
    "--instance-id",
    type=str,
    required=True,
    prompt=True,
    help="ID of the EC2 instance to terminate e.g. i-1234567890abcdef",
)
@option("-p", "--project", type=str, default="", help="Project name e.g /path/to/project")
def terminate(instance_id, project):
    logger.debug("ec2 terminate")

    if project != "":
        if not os_path.isabs(project):
            logger.error(f"Project path {project} is not absolute")
            return
        if not os_path.exists(project):
            logger.error(f"Project {project} does not exist on disk")
            return

    logger.info(f"Terminating EC2 Instance {instance_id}")
    input("Press Enter to continue...")

    ec2_client = boto3_client("ec2")

    try:
        response = ec2_client.terminate_instances(InstanceIds=[instance_id])
        instances_data = response["TerminatingInstances"]
        if len(instances_data) == 0:
            print("Failed to terminate EC2 instance")
            return

        instance_id = instances_data[0]["InstanceId"]
        logger.debug(f"Terminated 1 EC2 instance with ID: {instance_id}")
        print(f"Terminated 1 EC2 instance with ID: {instance_id}")
    except Exception as e:
        msg = f"Failed to terminate EC2 instance due to {e}"
        logger.error(msg)
        print("Failed to launch EC2 instance")

    remove_instance(instance_id=instance_id, project=None if project == "" else project)


def launch_instances(
    launch_template_id: str,
    key_pair_name: str,
    instance_name: str,
    count: int,
) -> List[Instance]:
    if count < 1:
        logger.error("Count must be greater than 0")
        return []

    ec2_client = boto3_client("ec2")

    try:
        # Launch an EC2 instance using the specified launch template, security group, and key pair
        response = ec2_client.run_instances(
            LaunchTemplate={"LaunchTemplateId": launch_template_id},
            KeyName=key_pair_name,
            MinCount=count,
            MaxCount=count,
            TagSpecifications=[{"ResourceType": "instance", "Tags": [{"Key": "Name", "Value": instance_name}]}],
        )

        # Extract the instance ID from the response
        instances_data = response["Instances"]
        instance_ids = [instance["InstanceId"] for instance in instances_data]

        instances: List[Instance] = []
        for instance_id in instance_ids:
            instances.append(Instance(id=instance_id, ip=""))

        logger.debug(f"Launched {len(instances)} instance: {instances}")
        return instances
    except Exception as e:
        msg = f"Failed to launch EC2 Instances due to {e}"
        logger.error(msg)
        return []


def create_and_associate_eip(instance_id: str) -> str:
    if instance_id is None or instance_id == "":
        logger.error("Instance ID is required")
        return ""

    ec2_client = boto3_client("ec2")

    # Allocate a new Elastic IP Address
    allocation = ec2_client.allocate_address(Domain="vpc")
    eip_address = allocation["PublicIp"]
    logger.debug(f"Allocated EIP: {eip_address}")

    # Associate the EIP with the specified EC2 instance
    allocation_id = allocation["AllocationId"]
    ec2_client.associate_address(InstanceId=instance_id, AllocationId=allocation_id)
    logger.debug(f"Associated EIP {eip_address} with EC2 Instance {instance_id}")

    return eip_address



import os

import pytest

from pd.conv.conv import FileSource, generate_file_sources


def test_file_source():
    fs = FileSource(name="test", type="txt", path="/path/to")
    assert str(fs) == "/path/to/test.txt"
    assert fs.name == "test"
    assert fs.type == "txt"
    assert fs.path == "/path/to"


def test_file_source_immutability():
    fs = FileSource(name="test", type="txt", path="/path/to")
    with pytest.raises(AttributeError):
        fs.name = "new_name"


@pytest.mark.parametrize(
    "src_filepath, dst_format, expected",
    [
        (
            "/path/to/file.txt",
            "pdf",
            (
                FileSource(name="file", type=".txt", path="/path/to"),
                FileSource(name="file", type="pdf", path="/path/to"),
            ),
        ),
        (
            "/home/user/doc.docx",
            "txt",
            (
                FileSource(name="doc", type=".docx", path="/home/user"),
                FileSource(name="doc", type="txt", path="/home/user"),
            ),
        ),
        (
            "relative/path/image.jpg",
            "png",
            (
                FileSource(name="image", type=".jpg", path="relative/path"),
                FileSource(name="image", type="png", path="relative/path"),
            ),
        ),
    ],
)
def test_generate_file_sources(src_filepath, dst_format, expected):
    result = generate_file_sources(src_filepath, dst_format)
    assert result == expected
    assert isinstance(result[0], FileSource)
    assert isinstance(result[1], FileSource)


def test_generate_file_sources_with_multiple_dots():
    result = generate_file_sources("/path/to/file.with.dots.txt", "pdf")
    assert result[0].name == "file.with.dots"
    assert result[0].type == ".txt"
    assert result[1].name == "file.with.dots"
    assert result[1].type == "pdf"


def test_generate_file_sources_with_no_extension():
    result = generate_file_sources("/path/to/file_without_extension", "pdf")
    assert result[0].name == "file_without_extension"
    assert result[0].type == ""
    assert result[1].name == "file_without_extension"
    assert result[1].type == "pdf"



from loguru import logger
from pydub import AudioSegment


def mp3(src: str, dst: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == "wav":
        mp3_to_wav(src, dst)
    elif dst_format == "m4a":
        mp3_to_m4a(src, dst)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def mp3_to_wav(src: str, dst: str) -> None:
    audio = AudioSegment.from_mp3(src)
    audio.export(dst, format="wav")
    return


def mp3_to_m4a(src: str, dst: str) -> None:
    audio = AudioSegment.from_mp3(src)
    audio.export(dst, format="ipod")
    return


def m4a(src: str, dst: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == "wav":
        m4a_to_wav(src, dst)
    elif dst_format == "mp3":
        m4a_to_mp3(src, dst)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def m4a_to_wav(src: str, dst: str) -> None:
    audio = AudioSegment.from_file(src, format="m4a")
    audio.export(dst, format="wav")
    return


def m4a_to_mp3(src: str, dst: str) -> None:
    audio = AudioSegment.from_file(src, format="m4a")
    audio.export(dst, format="mp3")
    return



from collections import namedtuple
from os import path as os_path

from loguru import logger
from moviepy.editor import AudioFileClip, VideoFileClip
from pydub import AudioSegment

Metadata = namedtuple("Metadata", ["fps", "width", "height"])


def webm(src: str, dst: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == "mp3":
        try:
            webm_video_to_mp3(src, dst)
        except Exception as e:
            logger.debug(f"Failed to convert {src} to {dst} as video, retrying as audio")
            webm_audio_to_mp3(src, dst)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def webm_video_to_mp3(src: str, dst: str) -> None:
    video = VideoFileClip(src)
    audio = video.audio
    audio.write_audiofile(dst)
    return


def webm_audio_to_mp3(src: str, dst: str) -> None:
    audio = AudioFileClip(src)
    audio.write_audiofile(dst)
    return


def mov(src: str, dst: str, quality: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == "mp3":
        try:
            mov_video_to_mp3(src, dst)
        except Exception as _:
            logger.debug(f"Failed to convert {src} to {dst} as video, retrying as audio")
            mov_audio_to_mp3(src, dst)
    elif dst_format == "gif":
        mov_to_gif(src, dst, quality)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def mov_video_to_mp3(src: str, dst: str) -> None:
    video = VideoFileClip(src)
    audio = video.audio
    audio.write_audiofile(dst)
    return


def mov_audio_to_mp3(src: str, dst: str) -> None:
    audio = AudioFileClip(src)
    audio.write_audiofile(dst)
    return

def mov_to_gif(src: str, dst: str, quality: str) -> None:
    metadata = get_metadata(src)
    logger.info(f"{src} has {metadata.width}x{metadata.height} resolution at {metadata.fps} fps")

    target_fps = get_target_fps(quality)
    target_width, target_height = get_target_res(metadata, quality)

    logger.info(f"Converting {src} to {dst} at {target_width}x{target_height} with {target_fps} fps")

    clip = VideoFileClip(filename=src, target_resolution=(target_height, target_width))
    clip.write_gif(dst, fps=target_fps)
    return


def mp4(src: str, dst: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == "mp3":
        mp4_to_mp3(src, dst)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def mp4_to_mp3(src: str, dst: str) -> None:
    video = VideoFileClip(src)
    audio = video.audio
    audio.write_audiofile(dst)
    return

def get_metadata(src: str) -> Metadata:
    with VideoFileClip(src) as clip:
        return Metadata(clip.fps, clip.size[0], clip.size[1])

def get_target_fps(quality: str) -> float:
    match quality:
        case "low":
            return 1
        case "medium":
            return 3
        case "high":
            return 5
        case _:
            return 3

def get_target_res(md: Metadata, quality: str) -> tuple[float, float]:
    scale = 1.0
    match quality:
        case "low":
            scale = 0.2
        case "medium":
            scale = 0.5
        case "high":
            scale = 0.75
        case _:
            scale = 0.5

    return int(md.width * scale), int(md.height * scale)



from os import getcwd
from os import path as os_path
from typing import Dict
from uuid import uuid4

from click import Path, argument, group, option
from loguru import logger
from moviepy.editor import ImageClip
from PIL import Image
from pydantic import BaseModel

from .audio import m4a, mp3
from .epub import epub
from .text import txt
from .video import mov, mp4, webm


class FileSource(BaseModel):
    name: str
    ext: str
    path: str

    _filename: str | None = None

    @property
    def filename(self):
        if not self._filename:
            self._filename = f"{self.name}.{self.ext}"
        return self._filename

    def __repr__(self):
        return f"FileSource(name={self.name}, ext={self.ext}, path={self.path})"

    class Config:
        frozen = True


@group(name="conv", help="Convert a file")
def conv():
    pass


@conv.command(help="Convert an image or multiple images into another format")
@option("-f", "--format", type=str, required=True, prompt=True, help="Format to convert to (e.g. jpg, mp4)")
@option("-d", "--duration", type=int, default=0, help="Duration of output file in seconds (e.g. 30)")
@argument("filepaths", nargs=-1, type=Path(exists=True), required=True)
def image(format: str, duration: int, filepaths: tuple[str, ...]):
    for input_path in filepaths:
        src_file, dst_file = generate_file_sources(input_path, format)

        logger.info(f"Converting {src_file.filename} to {dst_file.filename}")

        match dst_file.ext:
            case "jpg" | "jpeg" | "png" | "webp":
                with Image.open(src_file.path) as image:
                    image.save(dst_file.path, format.upper())
            case "mp4":
                clip = ImageClip(src_file.path, duration=duration)
                clip.write_videofile(dst_file.path, fps=24, codec="libx264")
            case _:
                logger.error(f"Unsupported format {dst_file.ext}")

    logger.info(f"Converted {len(filepaths)} image(s)")


@conv.command(help="Convert an audio to another format")
@option("-f", "--format", type=str, required=True, prompt=True, help="Format to convert to (e.g. wav)")
@argument("filepaths", nargs=-1, type=Path(exists=True), required=True)
def audio(format: str, filepaths: tuple[str, ...]):
    for filepath in filepaths:
        src_file, dst_file = generate_file_sources(filepath, format)

        logger.info(f"Converting {src_file.path} to {dst_file.path}")

        match src_file.ext:
            case "m4a":
                m4a(src_file.path, dst_file.path)
            case "mp3":
                mp3(src_file.path, dst_file.path)
            case _:
                logger.error(f"Unsupported file {src_file.ext}")


@conv.command(help="Convert a video to another format")
@option("-f", "--format", type=str, required=True, prompt=True, help="Format to convert to (e.g. mp3)")
@option("-q", "--quality", type=str, required=False, default="medium", help="Quality of output file (e.g. low, medium, high)")
@argument("filepaths", nargs=-1, type=Path(exists=True), required=True)
def video(format: str, quality: str, filepaths: tuple[str, ...]):
    for filepath in filepaths:
        src_file, dst_file = generate_file_sources(filepath, format)

        logger.info(f"Converting {src_file.path} to {dst_file.path}")

        match src_file.ext:
            case "mp4":
                mp4(src_file.path, dst_file.path)
            case "webm":
                webm(src_file.path, dst_file.path)
            case "mov":
                mov(src_file.path, dst_file.path, quality)
            case _:
                logger.error(f"Unsupported file {src_file.ext}")


@conv.command(help="Convert text into another format")
@option("-v", "--value", type=str, required=True, prompt=True, help="Text to convert")
@option("-f", "--format", type=str, required=True, prompt=True, help="Format to convert to (e.g. mp3)")
@option(
    "-o",
    "--options",
    type=str,
    required=False,
    default="voice=echo",
    help="Options to use for conversion (e.g. voice=alloy)",
)
def text(value: str, format: str, options: str):
    logger.debug("conv text")

    dst_format = format
    dst_path = f"{str(uuid4())}.{dst_format}"

    logger.debug(f"Converting {value} to {format} at ./{dst_path}")

    opts: Dict[str, str] = {k: v.lower() for k, v in [o.split("=") for o in options.split(",")]}

    txt(value, dst_path, opts)
    return


@conv.command(help="Convert a book into another format")
@option("-p", "--path", type=str, required=True, prompt=True, help="Path to the book file (e.g. /path/to/file)")
@option("-f", "--format", type=str, required=True, prompt=True, help="Format to convert to (e.g. pdf)")
def book(path: str, format: str) -> None:
    logger.debug("conv book")

    if not os_path.isabs(path):
        path = os_path.join(getcwd(), path)

    if not os_path.exists(path):
        print(f"Book {path} does not exist")
        return

    splits = path.split(".")

    src_format = splits[-1]
    src_filename = ".".join(splits[:-1])
    src_path = path

    dst_format = format
    dst_path = f"{src_filename}.{dst_format}"

    logger.debug(f"Converting {src_path} to {dst_path}")

    if src_format == "epub":
        epub(src_path, dst_path)
    else:
        logger.error(f"Unsupported file {src_format}")
    return


def generate_file_sources(src_filepath: str, dst_format: str) -> tuple[FileSource]:
    if not os_path.isabs(src_filepath):
        src_filepath = os_path.join(getcwd(), src_filepath)

    if not os_path.exists(src_filepath):
        logger.error(f"File {src_filepath} does not exist")
        raise FileNotFoundError

    src_filename, src_format = os_path.splitext(os_path.basename(src_filepath))
    src_format = src_format[1:]
    src_dirpath = os_path.dirname(src_filepath)
    src_filepath = src_filepath

    return (
        FileSource(name=src_filename, ext=src_format, path=src_filepath),
        FileSource(name=src_filename, ext=dst_format, path=os_path.join(src_dirpath, f"{src_filename}.{dst_format}")),
    )



from fastapi import HTTPException
from fastapi import status


def ErrAuth(detail: str) -> HTTPException:
    return HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail=detail,
        headers={'WWW-Authenticate': 'Bearer'},
    )


def ErrInvalid(detail: str) -> HTTPException:
    return HTTPException(
        status_code=status.HTTP_400_BAD_REQUEST,
        detail=detail,
    )



from user_agents import parse


def parse_user_agent(user_agent: str) -> str:
    if user_agent:
        user_agent = parse(user_agent)
        browser = f"{user_agent.browser.family} {user_agent.browser.version_string}"
        return browser
    return "Unknown"



from fastapi import Depends
from fastapi import HTTPException
from httpx import AsyncClient
from loguru import logger
from pydantic import BaseModel
from sqlalchemy.orm import Session
from starlette.requests import Request
from starlette.status import HTTP_401_UNAUTHORIZED

from app.lib.db import create_user
from app.lib.db import get_db
from app.lib.db import read_user_by_email
from app.lib.errors import ErrAuth
from app.lib.models import UserCreate


class TokenInfo(BaseModel):
    user_id: str
    email: str
    verified: bool = False
    token: str
    expires_in: int


async def get_user(token: str) -> TokenInfo:
    """Verifies the Google OAuth2 token and returns the user data. If expired or invalid, raises 401"""
    async with AsyncClient() as client:
        token_info = await client.get(
            url="https://www.googleapis.com/oauth2/v1/tokeninfo",
            params={"access_token": token}
        )
        if token_info.status_code != 200:
            raise ErrAuth("Invalid token")

        token_data = token_info.json()
        if token_data.get("error"):
            raise ErrAuth("Invalid token")

        if token_data.get("verified_email") is False:
            raise ErrAuth("Unverified email")

        data = token_info.json()
        return TokenInfo(
            user_id=data["user_id"],
            email=data["email"],
            verified=data["verified_email"],
            token=token,
            expires_in=data["expires_in"],
        )


async def get_current_user(
        request: Request,
        conn: Session = Depends(get_db),
) -> TokenInfo:
    auth_header = request.headers.get('Authorization')
    if auth_header is None:
        raise HTTPException(
            status_code=HTTP_401_UNAUTHORIZED, detail="Missing token")

    # Assuming the format is "Bearer <token>"
    token = auth_header.split(" ")[1]
    token_data = await get_user(token)

    db_user = read_user_by_email(conn=conn, email=token_data.email)

    if not db_user:
        # Create user if not found in DB (first time login)
        new_user = await get_profile_info(token_data)
        db_user = create_user(conn=conn, user=new_user)

        if not db_user:
            raise ErrAuth("Unregistered")

    # Check if user is deactivated
    if not db_user.is_active:
        logger.debug(f"User is deactivated: {db_user.email}")
        raise ErrAuth("Deactivated")

    return token_data


async def get_profile_info(token_info: TokenInfo) -> UserCreate:
    """Fetches the user's Google Info a UserBase Object. If expired or invalid, raises 401"""
    if not token_info.verified:
        raise ErrAuth("Unverified email")
    if token_info.expires_in < 10:
        raise ErrAuth("Token expired")

    async with AsyncClient() as client:
        user_info = await client.get(
            url="https://www.googleapis.com/oauth2/v1/userinfo?alt=json",
            params={"access_token": token_info.token}
        )
        if user_info.status_code != 200:
            raise ErrAuth("Invalid token")

        user_data = user_info.json()
        if user_data.get("error"):
            raise ErrAuth("Invalid token")

        logger.debug(f"UserInfo: {user_info.json()}")

        return UserCreate(
            id=user_data["id"],
            email=user_data["email"],
            verified=True,
            name=user_data["name"],
            picture=user_data["picture"],
        )



from fastapi import APIRouter
from fastapi import Depends
from fastapi import Request
from pydantic import BaseModel
from sqlalchemy.orm import Session

from app.lib.db import create_activity
from app.lib.db import get_db
from app.lib.models import ActivityCreate
from app.lib.utils import parse_user_agent

router = APIRouter(
    prefix="/analytics",
    tags=["analytics"],
)


class ClickReq(BaseModel):
    button_name: str


@router.post("/click", response_model=None)
async def click(
        *,
        conn: Session = Depends(get_db),
        req: ClickReq,
        request: Request,
):
    ip = request.headers.get("X-Forwarded-For", request.client.host).split(",")[0]
    browser = parse_user_agent(request.headers.get("User-Agent"))
    create_activity(conn, ActivityCreate(type="click", where=req.button_name, source=ip, browser=browser))



from os import path as os_path

from pydantic import BaseModel

PD_CONFIG_FILE = os_path.expanduser('~/.pdconfig.json')


class UserProfile(BaseModel):
    """Each user is uniquely identified by their username"""
    name: str | None = None  # User's name
    email: str | None = None  # User's email
    github: str | None = None  # GitHub username
    ssh_key: str | None = None  # SSH key path


class EnvVar(BaseModel):
    """Each environment variable is uniquely identified by its name"""
    key: str  # Environment variable name
    value: str  # Environment variable value


class Instance(BaseModel):
    """Each instance is uniquely identified by its ID"""
    id: str  # Instance ID
    ip: str  # Instance IP (e.g. Elastic IP)


class Project(BaseModel):
    """Each project is uniquely identified by its full path on disk"""
    type: str  # Type of project (e.g. react, fastapi)
    name: str  # Name of the project (e.g. project)
    path: str  # Full path on disk (e.g. /path/to/project)
    remote: str | None = None  # Remote path e.g. RE-LABS/ReX/rexhttp
    env: list[EnvVar] = []  # Environment variables
    pods: list[str] = 0  # Instances (e.g. EC2 instances)
    domain: str | None = None  # Domain name (e.g. example.com)


class Defaults(BaseModel):
    aws_launch_template: str  # Name of the default EC2 launch template
    aws_pem_key: str  # Path of the default EC2 PEM key
    aws_region: str  # Default EC2 region


class Pod(BaseModel):
    cluster_name: str | None
    instance_name: str
    instance_type: str
    key_name: str  # SSH key for the pod
    vpc_id: str
    subnet_id: str
    launch_template: str
    security_groups: dict[str, str]  # Name to Security Group
    private_dns: dict[str, str]  # Name to IP
    public_dns: dict[str, str] | None  # Name to IP


class AWS(BaseModel):
    access_key_id: str
    secret_access_key: str
    pods: dict[str, Pod]


class PDConfig(BaseModel):
    profile: UserProfile
    projects: list[Project]
    defaults: Defaults
    aws: AWS



def parse_str_list_int(ctx, param, value) -> list[int]:
    if not value:
        return []
    delim: str | None = None
    if ',' in value:
        delim = ','
    elif 'x' in value:
        delim = 'x'
    elif ' ' in value:
        delim = ' '

    if not delim:
        return [int(value)]

    return [int(s) for s in value.split(delim)]


from click import group
from click import option
from loguru import logger
from pytubefix import YouTube

from .libgen import download_book
from .youtube import get_script


@group(name='down', help="Download from the internet")
def down():
    pass


@down.command(help="Download a YouTube video")
@option('-l', '--link', type=str, required=True, prompt=True,
        help="Link to the YouTube video (e.g. https://www.youtube.com/watch?v=...)")
@option('-f', '--format', type=str, required=True, prompt=True,
        help="Format to download as (e.g. mp4, mp3, txt)")
def youtube(link: str, format: str):
    logger.debug("down video")

    if format == 'mp4':
        yt = YouTube(link)
        stream = yt.streams.filter(progressive=True, file_extension='mp4').order_by('resolution').desc().first()
        stream.download()
    elif format == 'mp3':
        yt = YouTube(link)
        stream = yt.streams.get_audio_only()
        stream.download(mp3=True)
    elif format == 'webm':
        yt = YouTube(link)
        stream = yt.streams.filter(only_audio=True).order_by('abr').desc().first()
        stream.download()
    elif format == 'txt':
        title, script = get_script(link)
        with open(f"{title}.txt", 'w') as f:
            f.write(script)
    else:
        logger.error(f"Unsupported format {format}")


@down.command(help="Download a book from Libgen")
@option('-n', '--name', type=str, required=True, prompt=True,
        help="Author name")
@option('-t', '--title', type=str, required=True, prompt=True,
        help="Book title")
def libgen(name: str, title: str):
    logger.debug("down book")
    download_book(name, title)



from sys import stderr
from sys import stdout

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.staticfiles import StaticFiles
from loguru import logger
from uvicorn.middleware.proxy_headers import ProxyHeadersMiddleware

from app.api import analytics
from app.api import home
from app.api import users
from app.lib.db import create_db_and_tables
from app.lib.settings import settings

logger.remove(0)
logger.add(stdout, level="DEBUG")
logger.add(stderr, level="ERROR")
logger.add("logs/app.log", level="INFO", rotation="1 day", retention="1 week")

app = FastAPI()

logger.debug(f"Environment: {settings.model_dump()}")


@app.on_event("startup")
async def startup_event():
    create_db_and_tables()


app.mount("/public", StaticFiles(directory="public"), name="public")

app.add_middleware(ProxyHeadersMiddleware, trusted_hosts="*")  # For Nginx

app.include_router(analytics.router)
app.include_router(users.router)
app.include_router(home.router)  # Must be last

origins = [
    # Replace this with your domain
    "*",
]

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)



from glob import glob
from os import makedirs
from os import path as os_path
from typing import List, Tuple

from click import Path, argument, group, option
from loguru import logger
from PIL import Image, ImageDraw, ImageFilter

from pd.__model__.utils import parse_str_list_int


@group(name="edit", help="Edit a file")
def edit():
    pass

@edit.command(help="Size an image")
@argument("filepaths", nargs=-1, type=Path(exists=True), required=True)
def size(filepaths: Tuple[str, ...]):
    for path in filepaths:
        img = Image.open(path).convert("RGBA")
        width, height = img.size
        filename = os_path.basename(path)
        logger.info(f"Image {filename}: {width}x{height} pixels")


@edit.command(help="Round an image")
@option("-p", "--path", type=str, required=True, prompt=True, help="Input image file path")
@option("-r", "--radius", type=str, default="20%", show_default=True, help="Radius (pixels or percentage)")
def round(path: str, radius: str):
    img = Image.open(path).convert("RGBA")
    width, height = img.size

    radius_px = radius_to_pixels(radius, width, height)
    rounded_img = round_corners(img, radius_px)

    file_name, file_ext = os_path.splitext(path)
    new_file_name = f"{file_name}_rounded{width}x{height}.png"

    rounded_img.save(new_file_name, "PNG")
    logger.info(f"Rounded image saved as {new_file_name}")


@edit.command(help="Resize image(s)")
@option("-s", "--size", type=int, help="Size of longer side (pixels)")
@option("-d", "--dimensions", callback=parse_str_list_int, type=str, help="Dimensions in pixels e.g. 512x512")
@option("-o", "--output", type=str, default="", help="Output folder path (optional)")
@option("-n", "--no-suffix", is_flag=True, help="Don't add size suffix to output filename")
@argument("filepaths", nargs=-1, type=Path(exists=True), required=True)
def resize(size: int, dimensions: list[int], output: str, no_suffix: bool, filepaths: Tuple[str, ...]):
    if len(dimensions) != 2 and not size:
        logger.error("Please provide either a size or two dimensions")
        return

    if output:
        makedirs(output, exist_ok=True)

    for input_file in filepaths:
        img = Image.open(input_file).convert("RGBA")
        width, height = img.size
        logger.info(f"Original size: {width}x{height}")

        if dimensions:
            new_dims = (dimensions[0], dimensions[1])
        else:
            longer_side = max(width, height)
            scale = size / longer_side
            new_dims = (int(width * scale), int(height * scale))

        img = high_quality_resize(img, new_dims)

        file_name, file_ext = os_path.splitext(os_path.basename(input_file))
        if no_suffix:
            new_file_name = f"{file_name}{file_ext}"
        else:
            new_file_name = f"{file_name}_{new_dims[0]}x{new_dims[1]}{file_ext}"

        if output:
            new_file_path = os_path.join(output, new_file_name)
        else:
            new_file_path = os_path.join(os_path.dirname(input_file), new_file_name)

        img.save(new_file_path, format=file_ext[1:].upper())
        logger.info(f"Resized image saved as {new_file_path}")

    logger.info(f"Resized {len(filepaths)} image(s)")


@edit.command(help="Scale an image")
@option("-p", "--path", type=str, required=True, prompt=True, help="Input image file path")
@option("-s", "--scale", type=float, required=True, prompt=True, help="Scale factor")
def scale(path: str, scale: float):
    img = Image.open(path).convert("RGBA")

    size = (int(img.size[0] * scale), int(img.size[1] * scale))
    img = high_quality_resize(img, size)

    file_name, file_ext = os_path.splitext(path)
    new_file_name = f"{file_name}_{size[0]}x{size[1]}.png"

    img.save(new_file_name, "PNG")
    logger.info(f"Scaled image saved as {new_file_name}")


@edit.command(help="Generate logos from an image")
@option("-p", "--path", type=str, required=True, prompt=True, help="Input image file path")
@option(
    "-t",
    "--template",
    type=str,
    default="Generic=(16, 32, 48, 128, 256, 512, 1024)",
    show_default=True,
    help="Template for logo sizes",
)
@option("-s", "--sizes", callback=parse_str_list_int, type=str, default="", show_default=True, help="Sizes for icon")
@option("-r", "--radius", type=str, default="", show_default=True, help="Radius (pixels or percentage)")
def logos(path: str, template: str, sizes: List[int], radius: str):
    logger.debug("logo")

    original_image = Image.open(path).convert("RGBA")
    width, height = original_image.size

    # Ensure the image is square
    if width != height:
        logger.warning("Image is not square. Cropping to make it square.")
        size = min(width, height)
        left = (width - size) // 2
        top = (height - size) // 2
        right = left + size
        bottom = top + size
        original_image = original_image.crop((left, top, right, bottom))

    # Get sizes from template
    template = "Generic" if "Generic" in template else template
    logo_sizes = sizes if sizes else get_sizes_from_template(template)

    # Check if upscaling is needed
    max_size = max(logo_sizes)
    if original_image.size[0] < max_size:
        logger.info(f"Upscaling image to {max_size}x{max_size}")
        original_image = high_quality_resize(original_image, (max_size, max_size))

    if radius:
        radius_px = radius_to_pixels(radius, original_image.size[0], original_image.size[1])
        logger.info(f"Rounding corners with radius {radius_px}")
        original_image = round_corners(original_image, radius_px)

    # Generate all sizes
    for size in logo_sizes:
        resized_image = high_quality_resize(original_image, (size, size))

        # Generate output filename
        file_name, file_ext = os_path.splitext(path)
        output_path = f"{file_name}{size}{file_ext}"

        # Save the resized image
        resized_image.save(output_path)
        logger.debug(f"Generated logo: {output_path}")

    logo_sizes = [f"{size}x{size}" for size in logo_sizes]
    logger.info(f"{len(logo_sizes)} logos ({', '.join(logo_sizes)}) and saved in the same directory")


@edit.command(help="Generate favicon from an image")
@option("-p", "--path", type=str, required=True, prompt=True, help="Input image file path")
@option("-s", "--sizes", type=list, default=[16, 32, 48], show_default=True, help="Sizes for icon")
def favicon(path: str, sizes: List[int]):
    original_image = Image.open(path).convert("RGBA")

    icon_sizes = []
    for size in sizes:
        resized_image = high_quality_resize(original_image, (size, size))
        icon_sizes.append(resized_image)

    output_path = f"{os_path.splitext(path)[0]}.ico"
    icon_sizes[0].save(output_path, format="ICO", sizes=[(size, size) for size in sizes], append_images=icon_sizes[1:])

    logger.info(f"Icon created successfully: {output_path}")
    logger.info(f"Icon sizes: {sizes}")


def radius_to_pixels(radius: str, width: int, height: int) -> int:
    if not radius:
        return 0
    if radius.endswith("%"):
        percentage = float(radius[:-1]) / 100
        return int(min(width, height) * percentage / 2)
    else:
        return int(radius)


def round_corners(img: Image, radius: int) -> Image:
    circle = Image.new("L", (radius * 2, radius * 2), 0)
    draw = ImageDraw.Draw(circle)
    draw.ellipse((0, 0, radius * 2, radius * 2), fill=255)
    alpha = Image.new("L", img.size, 255)
    w, h = img.size
    alpha.paste(circle.crop((0, 0, radius, radius)), (0, 0))
    alpha.paste(circle.crop((0, radius, radius, radius * 2)), (0, h - radius))
    alpha.paste(circle.crop((radius, 0, radius * 2, radius)), (w - radius, 0))
    alpha.paste(circle.crop((radius, radius, radius * 2, radius * 2)), (w - radius, h - radius))
    img.putalpha(alpha)
    return img


def high_quality_resize(image: Image, size: Tuple[int, int]) -> Image:
    factor = 8
    img = image.copy()
    img.thumbnail((size[0] * factor, size[1] * factor), Image.LANCZOS)
    img = img.filter(ImageFilter.UnsharpMask(radius=2, percent=150, threshold=3))
    img = img.resize(size, Image.LANCZOS)
    return img


def get_sizes_from_template(template: str) -> List[int]:
    if template == "iOS":
        return [40, 60, 80, 120, 180]
    elif template == "Android":
        return [36, 48, 64, 72, 96, 144, 192]
    elif template == "macOS":
        return [16, 32, 128, 256, 512]
    elif template == "Chrome":
        return [16, 19, 32, 38, 48, 128]
    elif template == "Generic":
        return [16, 32, 48, 128, 256, 512, 1024]
    else:
        raise ValueError(f"Unsupported template: {template}")



from urllib.parse import unquote

from fastapi import APIRouter
from fastapi.templating import Jinja2Templates
from loguru import logger
from starlette.requests import Request
from starlette.responses import HTMLResponse

from app.lib.seo import metadata

router = APIRouter(
    prefix="",
    tags=["home"],
)

templates = Jinja2Templates(directory="ui")


@router.get("/{full_path:path}", response_class=HTMLResponse)
async def catch_all(request: Request, full_path: str):
    path = "/" + unquote(full_path)
    seo = metadata.get(path, metadata.get("/"))  # Default to home page metadata
    logger.debug(f"SEO metadata for {path}: {seo}")
    return templates.TemplateResponse("index.html", {
        "request": request,
        "data": {},
        "seo": seo,
    })



from datetime import timedelta
from math import ceil
from random import shuffle
from re import sub
from typing import Dict
from typing import List

from libgen_api import LibgenSearch
from loguru import logger
from numpy import repeat
from requests import get
from requests.exceptions import Timeout
from tqdm import tqdm


def are_books_same(
        title1: str,
        title2: str,
        year1: str,
        year2: str,
) -> bool:
    if are_texts_same(title1, title2):
        return True

    if are_texts_similar(title1, title2):
        return are_texts_similar(year1, year2)

    return False


def filter_search_results(
        books: List[Dict],
        limit: int,
) -> List[Dict]:
    # TODO: sort books first
    filtered_books: List[Dict] = []
    for i, book in enumerate(books):
        if len(filtered_books) >= limit:
            break

        skip = False
        for filtered_book in filtered_books:
            if are_books_same(
                    title1=book['Title'],
                    title2=filtered_book['Title'],
                    year1=book['Year'],
                    year2=filtered_book['Year'],
            ):
                err = f"Skipping {book['Title']} because {filtered_book['Title']}"
                logger.warning(err)
                skip = True
                break

        if skip:
            continue

        filtered_books.append(book)
    return filtered_books


def download_link(
        link: str,
        timeout: timedelta,
) -> bytes:
    # Download link using requests
    try:
        response = get(url=link, timeout=timeout.total_seconds())

        if response.status_code != 200:
            err = f"Failed to download {link} because server returned {response.status_code}"
            raise Exception(err)

        return bytes(response.content)
    except Timeout:
        err = f"Failed to download {link} because request timed out"
        raise TimeoutError(err)
    except Exception as e:
        err = f"Failed to download {link} because {e}"
        raise Exception(err)


def best_effort_download(
        links: List[str],
        timeout: timedelta,
        max_timeout: timedelta,
        max_retries: int,
) -> bytes:
    links = repeat(links, ceil(max_retries / len(links))).tolist()
    shuffle(links)
    for link in links:
        try:
            return download_link(link=link, timeout=timeout)
        except TimeoutError:
            err = f"Failed to download {link} because of timeout, retrying..."
            logger.warning(err)
            timeout = min(timeout * 2, max_timeout)
        except Exception as e:
            err = f"Failed to download {link} because {e}"
            logger.error(err)
            continue  # Try next link without increasing timeout
    return b''


def get_download_links(
        name: str,
        limit: int,
        titles: List[str],
) -> List[Dict]:
    s = LibgenSearch()
    results = s.search_author_filtered(
        query=name,
        filters={
            "Extension": "epub",
            "Language": "English",
        },
        exact_match=False,
    )

    for title in titles:
        results.extend(s.search_title_filtered(
            query=title,
            filters={
                "Extension": "epub",
                "Language": "English",
            },
            exact_match=False,
        ))

    logger.info(f"Found {len(results)} book by {name}")

    results = filter_search_results(
        books=results,
        limit=limit,
    )

    logger.info(f"Filtered {len(results)} book by {name} for title {titles[0]}")

    books = []
    for result in results:
        links_data: Dict = s.resolve_download_links(result)

        links: List[str] = []
        for link in links_data.keys():
            links.append(links_data[link])

        data = {
            "id": result['ID'],
            "metadata": {
                "title": result['Title'],
                "authors": result['Author'],
                "kind": result['Extension'],
            },
            "spider": {
                "type": "libgen",
                "links": links,
            }
        }

        books.append(data)

    return books


def download_book(
        name: str,
        title: str,
) -> None:
    logger.info(f"Downloading {title} by {name}")

    try:
        datas = get_download_links(
            name=name,
            limit=1,
            titles=[title],
        )
    except Exception as e:
        err = f"Failed to get books for {name} due to {e}"
        logger.error(err)
        return

    # Now download all books
    total = len(datas)
    count = 0
    for i in tqdm(range(total)):
        data = datas[i]

        # Check if links present
        if 'links' not in data['spider'] or len(data['spider']['links']) == 0:
            err = f"Skipping {data['metadata']['title']} because empty spider links"
            logger.error(err)
            continue

        filename = data['metadata']['title']
        extension = data['metadata']['kind']

        script_bytes = best_effort_download(
            links=data['spider']['links'],
            timeout=timedelta(seconds=10),
            max_timeout=timedelta(seconds=60),
            max_retries=5,
        )
        if len(script_bytes) == 0:
            err = f"Skipping {data['metadata']['title']} because empty script"
            logger.error(err)
            continue

        with open(f"{filename}.{extension}", 'wb') as f:
            f.write(script_bytes)

        count += 1

    logger.info(f"Downloaded {count} books for {name}")
    return


def simplify_text(title: str) -> str:
    title = title.strip()  # Remove leading/trailing whitespace
    title = title.lower()  # Convert to lowercase (if standardization requires)
    title = sub(r'\([^)]*\)', '', title)  # Remove parentheses and contents
    title = sub(r'(\w)-(\w)', r'\1 \2', title)  # Replace hyphens with spaces
    title = sub(r'[^\w\s]', '', title)  # Remove special characters
    title = sub(r'\s+', ' ', title)  # Replace multiple spaces with a single space
    return title


def are_texts_same(text1: str, text2: str) -> bool:
    text1 = simplify_text(text1)
    text2 = simplify_text(text2)
    return text1 == text2


def are_texts_similar(text1: str, text2: str) -> bool:
    text1 = simplify_text(text1)
    text2 = simplify_text(text2)
    return text1 == text2 or text1 in text2 or text2 in text1


if __name__ == '__main__':
    download_book(
        name="Alex Hormozi",
        title="$100M Offers",
    )



from re import sub as re_sub

from loguru import logger
from pytube import YouTube
from youtube_transcript_api import YouTubeTranscriptApi


def get_script(url: str) -> (str, str):
    yt = YouTube(url)
    title = re_sub(r'[\/:*?"<>|]', '', yt.title)

    if 'youtube.com' in url:
        url = url.split('=')[-1]

    subtitles = YouTubeTranscriptApi.get_transcript(url)

    script = " ".join([f"{sub['text']}" for sub in subtitles])
    return title, script


if __name__ == '__main__':
    link = f"https://www.youtube.com/watch?v=DxREm3s1scA"
    t, s = get_script(link)
    logger.info(f"{t}: {s}")
